/*
 *	Copyright 1988 by Rayan S. Zachariassen, all rights reserved.
 *	This will be free software, but only when it is finished.
 */

/* LINTLIBRARY */

#include "mailer.h"
#include "search.h"
#include "io.h"

/*
 * In-core database maintenance.
 *
 * Note that the keys are stashed forever because of the symbol() lookup
 * thing; this puts a practical limitation on the usefullness of this.
 * If people start using this seriously we need to rethink the key->hashid
 * mechanism.
 */


struct sptree *
open_core(sip)
	struct search_info *sip;
{
	if (sip->subtype == NULL)
		return NULL;
	return (struct sptree *)sip->subtype;
}

/*
 * Search an incore database for a key.
 */

struct conscell *
search_core(sip)
	struct search_info *sip;
{
	struct sptree *db;
	struct spblk *spl;
	struct conscell *tmp;

	if ((db = open_core(sip)) == NULL)
		return NULL;
	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL)
		return NULL;
	if (spl->data == NULL)
		return newstring((u_char *)strnsave("", 0));
	return newstring((u_char *)strnsave((char *)spl->data, strlen(spl->data)));
}

/*
 * Free any information stored in this database.
 */

int
icfreedata(spl)
	struct spblk *spl;
{
	if (spl->data)
		(void) free((char *)spl->data);
	return 0;
}

void
close_core(sip)
	struct search_info *sip;
{
	struct sptree *db;
	extern struct sptree *spt_loginmap;
	extern void sp_null();

	if ((db = open_core(sip)) == NULL)
		return;
	if (db != spt_loginmap)
		sp_scan(icfreedata, (struct spblk *)NULL, db);
	sp_null(db);
}

/*
 * Add the indicated key/value pair to the database.
 */

int
add_core(sip, value)
	struct search_info *sip;
	char *value;
{
	struct sptree *db;
	struct spblk *spl;
	int oval;
	extern int stickymem;

	if ((db = open_core(sip)) == NULL)
		return EOF;

	oval = stickymem;
	stickymem = MEM_MALLOC;
	if (*value == '\0')
		value = NULL;
	else
		value = strnsave(value, strlen(value));
	stickymem = oval;

	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL)
		sp_install(symbol(sip->key), value, 0, db);
	else {
		if (spl->data != NULL)
			(void) free((char *)spl->data);
		spl->data = (u_char *)value;
	}

	return NULL;
}

/*
 * Remove the indicated key from the database.
 */

int
remove_core(sip)
	struct search_info *sip;
{
	struct sptree *db;
	struct spblk *spl;

	if ((db = open_core(sip)) == NULL)
		return EOF;
	if ((spl = sp_lookup(symbol(sip->key), db)) == NULL) {
		fprintf(stderr, "remove_core: no such key as \"%s\"!\n",
				sip->key);
		return EOF;
	}
	if (spl->data != NULL)
		(void) free((char *)spl->data);
	sp_delete(spl, db);
	return NULL;
}

/*
 * Print the database.
 */

static FILE *pcfp;

int
icprintdata(spl)
	struct spblk *spl;
{
	if (spl->data != NULL)
		fprintf(pcfp, "%s\t%s\n", pname(spl->key), (char *)spl->data);
	else if ((int)(spl->data) < 0x10000)	/* uids are < 64k */
		fprintf(pcfp, "%s\t%d\n", pname(spl->key), spl->data);
	else
		fprintf(pcfp, "%s\n", pname(spl->key));
	return 0;
}

void
print_core(sip, outfp)
	struct search_info *sip;
	FILE *outfp;
{
	struct sptree *db;

	if ((db = open_core(sip)) == NULL)
		return;
	pcfp = outfp;
	sp_scan(icprintdata, (struct spblk *)NULL, db);
	(void) fflush(outfp);
}

void
owner_core(sip, outfp)
	struct search_info *sip;
	FILE *outfp;
{
	(void) fprintf(outfp, "%d\n", getuid());
	(void) fflush(outfp);
}
